/*+ FlashLoaderDlg.cpp
*
******************************************************************************
*
*                        Trimble Navigation Limited
*                           645 North Mary Avenue
*                              P.O. Box 3642
*                         Sunnyvale, CA 94088-3642
*
******************************************************************************
*
*    Copyright  2005 Trimble Navigation Ltd.
*    All Rights Reserved
*
******************************************************************************
*
* Description:
*    This file implements the CFlashLoaderDlg class.
*            
* Revision History:
*    09-22-2005    Mike Priven
*                  Cleaned up and adapted for outside distribution.
*
*    xx-xx-2001    Sean Zhi
*                  Original
*
* Notes:
*
-*/

/*---------------------------------------------------------------------------*\
|                         I N C L U D E   F I L E S
\*---------------------------------------------------------------------------*/
#include "stdafx.h"
#include "FlashLoader.h"
#include "FlashLoaderDlg.h"
#include "Options.h"
#include "sq_demon.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/*---------------------------------------------------------------------------*\
|                       L O C A L   C O N S T A N T S
\*---------------------------------------------------------------------------*/
const CString m_str_ok_msg[] =
{
    "Init State",
    "ENQ Sent",
    "Force2Mon Successful",
    "Loading Demon",
    "Demon Load Cmd Sent",
    "Demon Load Cmd Ack",
    "Demon Running",
    "Erase Cmd Sent",
    "Erase Successful",
    "FW Load Cmd Sent",
    "FW Load Cmd Ack",
    "FW Load Complete",
};

const CString m_str_err_msg[] =
{
    "Init State...",
    "ENQ Sent...",
    "Force2Mon Failed...",
    "Loading Demon...",
    "Demon Load Cmd Sent...",
    "Demon Load Cmd Not Ack...",
    "Demon Not Running...",
    "Erase Cmd Sent...",
    "Erase Failed...",
    "FW Load Cmd Sent...",
    "FW Load Cmd Not Ack...",
    "FW Load Incomplete..."
};

WCHAR pwchLicenseKey[] =
{
    0x0043,	0x006F,	0x0070,	0x0079,	0x0072,	0x0069,	
    0x0067,	0x0068,	0x0074,	0x0020,	0x0028,	0x0063,	
    0x0029,	0x0020,	0x0031,	0x0039,	0x0039,	0x0034,	
    0x0020
};

int ID_COMMCTRL[NUM_PORTS] = 
{
    ID_COMMCTRL_1, ID_COMMCTRL_2, ID_COMMCTRL_3,  ID_COMMCTRL_4, ID_COMMCTRL_5, 
    ID_COMMCTRL_6, ID_COMMCTRL_7, ID_COMMCTRL_8,  ID_COMMCTRL_9, ID_COMMCTRL_10,
    ID_COMMCTRL_11,ID_COMMCTRL_12,ID_COMMCTRL_13, ID_COMMCTRL_14,
    ID_COMMCTRL_15,ID_COMMCTRL_16
};


/*---------------------------------------------------------------------------*\
|                 F O R W A R D   D E C L A R A T I O N S
\*---------------------------------------------------------------------------*/
DWORD WINAPI FlashProgrammingThread (LPVOID pParam);


/*---------------------------------------------------------------------------*\
|                         I N N E R   C L A S S
\*---------------------------------------------------------------------------*/
class CAboutDlg : public CDialog
{
public:
    CAboutDlg();
    
    //{{AFX_DATA(CAboutDlg)
    enum { IDD = IDD_ABOUT };
    //}}AFX_DATA
    
protected:
    //{{AFX_VIRTUAL(CAboutDlg)
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    virtual BOOL OnInitDialog();
    //}}AFX_VIRTUAL
    
    //{{AFX_MSG(CAboutDlg)
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
    //{{AFX_DATA_INIT(CAboutDlg)
    //}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CAboutDlg)
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
    //{{AFX_MSG_MAP(CAboutDlg)
    // No message handlers
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CAboutDlg::OnInitDialog()
{
    CString str;
    
    CDialog::OnInitDialog();
    
    GetDlgItem(IDC_TITLE)->SetWindowText(APP_NAME);
    str.Format("Version: %d.%02d",APP_VER_MAJOR,APP_VER_MINOR);
    GetDlgItem(IDC_VER)->SetWindowText(str);
    str.Format("Date: %02d/%02d/%d",APP_VER_MON,APP_VER_DAY,APP_VER_YEAR);
    GetDlgItem(IDC_DATE)->SetWindowText(str);
    
    return TRUE;
}


/*---------------------------------------------------------------------------*\
|                    M E T H O D   D E F I N I T I O N S
\*---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
Function:       <constructor>

  Description:    A constructor for this class. Called when a new object of this
  class is created.
  
    Parameters:     pParent - a pointer to the parent window - the window that
    created an object of this class.
    
      Return Value:   none
-----------------------------------------------------------------------------*/
CFlashLoaderDlg::CFlashLoaderDlg(CWnd* pParent /*=NULL*/)
: CDialog(CFlashLoaderDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CFlashLoaderDlg)
    m_str_filename = _T("");
    //}}AFX_DATA_INIT
    // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

/*-----------------------------------------------------------------------------
Function:       DoDataExchange

  Description:    Called by the framework to exchange and validate dialog data.
  Never call this function directly. It is called by the 
  UpdateData() member function
  
    Parameters:     pDX - a pointer to a CDataExchange object
    
      Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    
    DDX_Control(pDX, IDC_CHECK1,  m_checkboxs[0]);
    DDX_Control(pDX, IDC_CHECK2,  m_checkboxs[1]);
    DDX_Control(pDX, IDC_CHECK3,  m_checkboxs[2]);
    DDX_Control(pDX, IDC_CHECK4,  m_checkboxs[3]);
    DDX_Control(pDX, IDC_CHECK5,  m_checkboxs[4]);
    DDX_Control(pDX, IDC_CHECK6,  m_checkboxs[5]);
    DDX_Control(pDX, IDC_CHECK7,  m_checkboxs[6]);
    DDX_Control(pDX, IDC_CHECK8,  m_checkboxs[7]);
    DDX_Control(pDX, IDC_CHECK9,  m_checkboxs[8]);
    DDX_Control(pDX, IDC_CHECK10, m_checkboxs[9]);
    DDX_Control(pDX, IDC_CHECK11, m_checkboxs[10]);
    DDX_Control(pDX, IDC_CHECK12, m_checkboxs[11]);
    DDX_Control(pDX, IDC_CHECK13, m_checkboxs[12]);
    DDX_Control(pDX, IDC_CHECK14, m_checkboxs[13]);
    DDX_Control(pDX, IDC_CHECK15, m_checkboxs[14]);
    DDX_Control(pDX, IDC_CHECK16, m_checkboxs[15]);
    
    DDX_Control(pDX, IDC_EDIT_FILENAME_1,  m_edit_status[0]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_2,  m_edit_status[1]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_3,  m_edit_status[2]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_4,  m_edit_status[3]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_5,  m_edit_status[4]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_6,  m_edit_status[5]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_7,  m_edit_status[6]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_8,  m_edit_status[7]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_9,  m_edit_status[8]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_10, m_edit_status[9]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_11, m_edit_status[10]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_12, m_edit_status[11]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_13, m_edit_status[12]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_14, m_edit_status[13]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_15, m_edit_status[14]);
    DDX_Control(pDX, IDC_EDIT_FILENAME_16, m_edit_status[15]);
    
    //{{AFX_DATA_MAP(CFlashLoaderDlg)
    DDX_Control(pDX, IDC_PROGRESS,          m_progress);
    DDX_Text   (pDX, IDC_EDIT_ROM_FILENAME, m_str_filename);
    //}}AFX_DATA_MAP
}

/*-----------------------------------------------------------------------------
Function:       BEGIN_MESSAGE_MAP (theClass, baseClass)

Description:    Defines a message map for this class. Matches a windows message
                with a callback function. The framework will call this function
                once the message associated with it has been received.

Parameters:     theClass - specifies the name of the class whose message map
                this is
                baseClass - specifies the name of the base class of theClass

Return Value:   none
-----------------------------------------------------------------------------*/
BEGIN_MESSAGE_MAP(CFlashLoaderDlg, CDialog)
    //{{AFX_MSG_MAP(CFlashLoaderDlg)
    ON_WM_SYSCOMMAND()
    ON_WM_DESTROY()
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_BN_CLICKED(IDC_CHECK1,      OnCheck1)
    ON_BN_CLICKED(IDC_CHECK2,      OnCheck2)
    ON_BN_CLICKED(IDC_CHECK3,      OnCheck3)
    ON_BN_CLICKED(IDC_CHECK4,      OnCheck4)
    ON_BN_CLICKED(IDC_CHECK5,      OnCheck5)
    ON_BN_CLICKED(IDC_CHECK6,      OnCheck6)
    ON_BN_CLICKED(IDC_CHECK7,      OnCheck7)
    ON_BN_CLICKED(IDC_CHECK8,      OnCheck8)
    ON_BN_CLICKED(IDC_CHECK9,      OnCheck9)
    ON_BN_CLICKED(IDC_CHECK10,     OnCheck10)
    ON_BN_CLICKED(IDC_CHECK11,     OnCheck11)
    ON_BN_CLICKED(IDC_CHECK12,     OnCheck12)
    ON_BN_CLICKED(IDC_CHECK13,     OnCheck13)
    ON_BN_CLICKED(IDC_CHECK14,     OnCheck14)
    ON_BN_CLICKED(IDC_CHECK15,     OnCheck15)
    ON_BN_CLICKED(IDC_CHECK16,     OnCheck16)
    ON_BN_CLICKED(IDC_START,       OnStart)
    ON_BN_CLICKED(IDC_ABOUT,       OnAbout)
    ON_BN_CLICKED(IDC_FILECHOOSER, OnFileChooser)
    ON_BN_CLICKED(IDC_SETTINGS,    OnSettings)
    ON_WM_CLOSE()
    //}}AFX_MSG_MAP
    ON_BN_CLICKED(IDC_CLOSE, OnClose)
END_MESSAGE_MAP()

/*-----------------------------------------------------------------------------
Function:       BEGIN_EVENTSINK_MAP(CFlashLoaderDlg, CDialog)

Description:    Defines an event map for OnComm event

Parameters:     theClass - specifies the name of the class whose message map
                this is
                baseClass - specifies the name of the base class of theClass

Return Value:   none
-----------------------------------------------------------------------------*/
BEGIN_EVENTSINK_MAP(CFlashLoaderDlg, CDialog)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_1,  1, OnCommEvent_1,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_2,  1, OnCommEvent_2,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_3,  1, OnCommEvent_3,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_4,  1, OnCommEvent_4,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_5,  1, OnCommEvent_5,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_6,  1, OnCommEvent_6,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_7,  1, OnCommEvent_7,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_8,  1, OnCommEvent_8,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_9,  1, OnCommEvent_9,  VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_10, 1, OnCommEvent_10, VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_11, 1, OnCommEvent_11, VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_12, 1, OnCommEvent_12, VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_13, 1, OnCommEvent_13, VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_14, 1, OnCommEvent_14, VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_15, 1, OnCommEvent_15, VTS_NONE)
    ON_EVENT(CFlashLoaderDlg, ID_COMMCTRL_16, 1, OnCommEvent_16, VTS_NONE)
END_EVENTSINK_MAP()
      
/*-----------------------------------------------------------------------------
Function:       OnCancel

Description:    The framework calls this member function when the user clicks 
                the Cancel button or presses the ESC key in a modal or 
                modeless dialog box. 

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnCancel()
{
    // Here, we don't do anything. That way, pressing Esc in the main GUI
    // window has no effect
}

/*-----------------------------------------------------------------------------
Function:       OnInitDialog

Description:    Initialises the dialog box. This is a standard MFC function 
                called automatically after the dialog has been created but 
                before it is displayed.
  
Parameters:     none

Return Value:   specifies whether the application has set the input focus to 
                one of the controls in the dialog box
-----------------------------------------------------------------------------*/
BOOL CFlashLoaderDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    
    CString str;
    str.Format("%s v%d.%02d",APP_NAME,APP_VER_MAJOR,APP_VER_MINOR);
    SetWindowText(str);
    
    // IDM_ABOUTBOX must be in the system command range.
    ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
    ASSERT(IDM_ABOUTBOX < 0xF000);
    
    CMenu* pSysMenu = GetSystemMenu(FALSE);
    if (pSysMenu != NULL)
    {
        pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, "About...");
    }
    
    // Set the icon for this dialog.  The framework does this automatically
    // when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);			// Set big icon
    SetIcon(m_hIcon, FALSE);		// Set small icon
    
    m_progress.SetStep(1);
    m_progress.SetPos(0);
    
    for (int idx=0; idx<NUM_PORTS; idx++)
    {
        m_commctrl_created[idx]   = false;
        m_bool_port_selected[idx] = false;
        m_int_mon_state[idx]      = S_MON_INIT;
    }
    
    m_loading_started = false;
    m_int_romType     = FW_ROM;
    m_romfile         = NULL;
    
    // Get some settings from the registry
    m_str_filename    = AfxGetApp()->GetProfileString("RomFile","NameAndPath", "");
    m_strFwBaseAddr   = AfxGetApp()->GetProfileString("Misc","FwBaseAddr", DEFAULT_FW_ADDR);
    
    UpdateData(false);
    
    return TRUE;
}

/*-----------------------------------------------------------------------------
Function:       OnSysCommand

Description:    The framework calls this member function when the system menu
                is selected.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
    if ((nID & 0xFFF0) == IDM_ABOUTBOX)
    {
        CAboutDlg dlgAbout;
        dlgAbout.DoModal();
    }
    else
    {
        CDialog::OnSysCommand(nID, lParam);
    }
}

/*-----------------------------------------------------------------------------
Function:       OnClose
    
Description:    This function is automatically called when a close button in
                in the upper right corner of the window is clicked on
                (the title-bar button with a cross). 

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnClose() 
{
    if (m_loading_started &&
    (IDYES != AfxMessageBox("Firmware loading is in progress! Unit may not\
    recover if the loading process is interrupted.\n\nAre you sure you want to exit?", 
    MB_YESNO | MB_ICONSTOP | MB_DEFBUTTON1, 0)))
    {
        return;
    }
    
    m_loading_started = false;
    
    for (int i=0; i<NUM_PORTS; i++)
    {
        if (m_bool_port_selected[i] && m_commctrl[i].GetPortOpen())
        {
            m_commctrl[i].SetPortOpen(FALSE);
        }
    }
    
    if (m_romfile)
    {
        fclose(m_romfile);
    }
    DestroyWindow();
}

/*-----------------------------------------------------------------------------
Function:       OnDestroy

Description:    The framework calls this member function when the main dialog 
                is about to be destroyed.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnDestroy()
{
    WinHelp(0L, HELP_QUIT);
    CDialog::OnDestroy();
}

/*-----------------------------------------------------------------------------
Function:       OnPaint

Description:    The framework calls this member function when Windows or an 
                application makes a request to repaint a portion of an 
                applications window

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnPaint() 
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting
        
        SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
        
        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;
        
        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}

/*-----------------------------------------------------------------------------
Function:       OnQueryDragIcon

Description:    The system calls this to obtain the cursor to display while the 
                user drags the minimized window.

Parameters:     none

Return Value:   A doubleword value that contains a cursor or icon handle in 
                the low-order word.
-----------------------------------------------------------------------------*/
HCURSOR CFlashLoaderDlg::OnQueryDragIcon()
{
    return (HCURSOR) m_hIcon;
}

/*-----------------------------------------------------------------------------
Function:       OnAbout

Description:    This function is called when the About menu is selected.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnAbout() 
{
    CAboutDlg dlg;
    dlg.DoModal();
}

/*-----------------------------------------------------------------------------
Function:       OnSettings
    
Description:    This function is called when the Settings buttons is pushed.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnSettings() 
{
    COptions dlg(this);
    dlg.DoModal();	
}

/*-----------------------------------------------------------------------------
Function:       OnStart

Description:    This function is called when the Start button is pushed.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnStart() 
{
    DWORD thrdID;
    BOOL  bAtLeaseOnePortSelected;
    
    bAtLeaseOnePortSelected = false;
    
    m_progress.SetPos(0);
    
    for (int i=0; i<NUM_PORTS; i++)
    {
        if (m_checkboxs[i].GetCheck() == 1)
        {
            m_bool_port_selected[i] = true;
            
            if (m_commctrl[i].GetPortOpen()!=1)
            {
                m_commctrl[i].SetPortOpen(true);
            }
            
            bAtLeaseOnePortSelected = true;
        }
        else
        {
            m_bool_port_selected[i] = false;
        }
    }
    
    UpdateData();
    
    if ((m_romfile == NULL) || m_str_filename.IsEmpty())
    {
        if (!m_str_filename.IsEmpty())
        {
            if (!OpenRomFile())
            {
                AfxMessageBox("The ROM file could not be opened!");
                return;
            }
        }
        else
        {
            AfxMessageBox("Select the ROM file first.");
            return;
        }
    }
    
    if (!bAtLeaseOnePortSelected)
    {
        AfxMessageBox("Select COM port(s) first.");
        return;
    }
    
    GetDlgItem(IDC_START)->EnableWindow(false);
    
    if ((m_flashLoadingThread = CreateThread (NULL,
								0, 
                                (LPTHREAD_START_ROUTINE)FlashProgrammingThread,
                                this,
                                0,
                                &thrdID)) == NULL)
    {
        AfxMessageBox("Failed to create system resources.");
    }
    
}

/*-----------------------------------------------------------------------------
Function:       OnFileChooser

Description:    This function is called when file chooser buttong (...) is
                clicked.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnFileChooser() 
{
    CFileDialog dlgFileChooser(TRUE, "rom", "", 
        OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
        _T("ROM File (.rom)|*.rom|All Files (*.*)|*.*|"));

    dlgFileChooser.m_ofn.lpstrTitle = "Select ROM File...";
    
    if (!m_str_filename.IsEmpty())
    {
        dlgFileChooser.m_ofn.lpstrInitialDir =	m_str_filename;
    }
    
    if (dlgFileChooser.DoModal() == IDOK) 
    {
        m_str_filename = dlgFileChooser.GetPathName();
        
        UpdateData(false);
        
        if (!OpenRomFile())
        {
            AfxMessageBox("The ROM file could not be opened!");
        }
    }
}

/*-----------------------------------------------------------------------------
Function:       OpenRomFile

Description:    Opens the selected ROM file.
                clicked.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
bool CFlashLoaderDlg::OpenRomFile()
{
    if (m_romfile)
    {
        fclose (m_romfile);
    }
    
    if ((m_romfile = fopen (m_str_filename, "r")) != NULL)
    {
        return true;
    }
    else
    {
        m_str_filename = "";
        UpdateData(false);
        return false;
    }
}


/*-----------------------------------------------------------------------------
Function:       FlashProgrammingThread
    
Description:    This thread loads the ROM file into a memory buffer and
                sends to the target to program into FLASH.

Parameters:     pParam - a pointer to the parent window which is the main
                         dialog window class CFlashLoaderDlg.

Return Value:   none
-----------------------------------------------------------------------------*/
DWORD WINAPI FlashProgrammingThread (LPVOID pParam)
{
    FILE          *fp;
    CString       str = "Failed to load ROM file";
    long	      memBufferSize;
    long	      memBufferSizeDemon;
    unsigned long baseAddr = 0;
    long 	      idx;
    short 	      len;
    char          cmdLine[512];
    
    CFlashLoaderDlg *pParent = (CFlashLoaderDlg*) pParam;
    
    // Load the Demon S-record ROM (as text stringsinto the memory buffer. The number of the
    // loadable data bytes (firmware size) is returned. 
    memBufferSizeDemon = CUtil::LoadSrecStr (pRRDemon, pParent->m_memBufferDemon, DEMON_START_ADDR);
    
    switch (memBufferSizeDemon)
    {
        case ERR_MEM_ALLOC:  
            str = "Memory allocation error";
            break;
        case ERR_SREC_CHKSM: 
            str = "Demon file checksum invalid (S-record)";
            break;
        case ERR_SREC_ADDR:  
            str = "Demon file address value invalid (S-record)";
            break;
        case ERR_SREC_BASE:  
            str = "Demon file type/base address mismatch (S-record)\n\nClick Settings and make sure they are correct.";
            break;
        case ERR_SREC_SIZE:
            str = "Demon file size mismatch (S-record)\n\nClick Settings and make sure they are correct.";
            break;
        case ERR_SREC_FORMAT:
            str = "Demon file format invalid (S-record)";
            break;
        default:
            break;
    }
    
    // File parsing error occurred.
    if (memBufferSizeDemon < 0) 
    {
        AfxMessageBox(str);
        pParent->TermFlashLoadingThd();
        return -1;
    }
    
    fp = pParent->m_romfile;
    
    sscanf (pParent->m_strFwBaseAddr, "%08x", &baseAddr);
    
    if (baseAddr == 0)
    {
        AfxMessageBox("Invalid base address specified.\n\nClick Settings and \
            make sure correct base address is specified.");
        pParent->TermFlashLoadingThd();
        return -1;
    }
    
    // Load the S-record ROM file into the memory buffer. The number of the
    // loadable data bytes (firmware size) is returned. 
    memBufferSize = CUtil::LoadSrecFile (fp, pParent->m_memBuffer, baseAddr);
    
    switch (memBufferSize)
    {
        case ERR_MEM_ALLOC:  
            str = "Memory allocation error";
            break;
        case ERR_SREC_CHKSM: 
            str = "ROM file checksum invalid (S-record)";
            break;
        case ERR_SREC_ADDR:  
            str = "ROM file address value invalid (S-record)";
            break;
        case ERR_SREC_BASE:  
            str = "ROM file type/base address mismatch (S-record)\n\nClick Settings and make sure they are correct.";
            break;
        case ERR_SREC_SIZE:
            str = "ROM file size mismatch (S-record)\n\nClick Settings and make sure they are correct.";
            break;
        case ERR_SREC_FORMAT:
            str = "ROM file format invalid (S-record)";
            break;
        default:
            break;
    }
    
    // File parsing error occurred.
    if (memBufferSize < 0) 
    {
        AfxMessageBox(str);
        pParent->TermFlashLoadingThd();
        return -1;
    }
    
    // Save some settings to the system registry
    AfxGetApp()->WriteProfileString("RomFile","NameAndPath", pParent->m_str_filename);
    AfxGetApp()->WriteProfileString("Misc","FwBaseAddr", pParent->m_strFwBaseAddr);
    
    // Send the force-to-monitor mode command.
    // First, load the command into a buffer.
    
    str.Format(_T("%d,%C,%d,%d"),9600,(TCHAR)'O',8,1);
    pParent->SetPortParams(str);
    
    len = CUtil::GetForce2MonPkt (cmdLine);
    pParent->MonStateSet(S_MON_INIT);
    pParent->SendStr (cmdLine, len);
   	
    // Wait for the switch
    Sleep (400);
    
    // Change the COM port settings to match the monitor mode settings of the target
    str.Format(_T("%d,%C,%d,%d"),9600,(TCHAR)'N',8,1);
    pParent->SetPortParams(str);
    
    pParent->m_loading_started = true;
    
    // Send ENQ to the target and wait for ACK to establish communication
    len = CUtil::GetEnqPkt (cmdLine);
    pParent->MonStateSet(S_MON_ENTRY_CMD_SENT);	
    pParent->SendStr (cmdLine, len);
    pParent->MonStateCheck(S_MON_ENTRY_CMD_ACK, 20, 15);	 
    
    // Initiate Demon loading
    Sleep(100);
    pParent->MonStateSet(S_MON_ENTRY_CMD_CMP);	
    
    // Load the Demon code here...
    idx = 0;
    while (idx < memBufferSizeDemon)
    {
        int tempLen;
        static long actualProgress = memBufferSizeDemon/100;
        
        if ((idx + PACKET_SIZE) > memBufferSizeDemon)
        {
            tempLen = memBufferSizeDemon - idx;
        }
        else
        {
            tempLen = PACKET_SIZE;
        }
        pParent->MonStateSet(S_DEMON_CMD_SENT);
        len = CUtil::GetDemonProgramPkt (cmdLine, (char*)&(pParent->m_memBufferDemon[idx]), tempLen, DEMON_START_ADDR+idx);
        pParent->SendStr (cmdLine, len);
        
        idx += tempLen;
        
        // Update the progress control on the main window.
        if (idx > actualProgress)
        {
            pParent->m_progress.StepIt();
            actualProgress += memBufferSizeDemon/100;
        }
        
        // Check for the ACK
        pParent->MonStateCheck(S_DEMON_CMD_ACK, 5, 60);
    }
    pParent->MonStateSet(S_DEMON_CMD_CMP);	
    pParent->m_progress.SetPos(100);
    
    // Run demon code
    len = CUtil::GetStartDemonPkt (cmdLine);
    pParent->SendStr (cmdLine, len);
    Sleep(300);
    
    // Change the Demon port baud rate to 57600
    len = CUtil::GetDemonSetPortPkt (cmdLine, 57600);
    pParent->SendStr (cmdLine, len);
    Sleep(100);
    
    // Change the COM port settings to match the demon settings of the target
    str.Format(_T("%d,%C,%d,%d"),57600,(TCHAR)'N',8,1);
    pParent->SetPortParams(str);
    
    Sleep(600);
    
    // Erase the FLASH memory block corresponding to the selected firmware ROM type
    len = CUtil::GetFwErasePkt (cmdLine);
    pParent->MonStateSet(S_ERASE_CMD_SENT);	
    pParent->SendStr (cmdLine, len);
    
    // Wait for ACK
    pParent->MonStateCheck(S_ERASE_CMD_ACK, 500, 40);	
    
    // Initiate firmware loading
    pParent->m_progress.SetPos(0);
    
    idx = 0;
    while (idx < memBufferSize)
    {
        int tempLen;
        static long actualProgress = memBufferSize/100;
        
        if ((idx + PACKET_SIZE) > memBufferSize)
        {
            tempLen = memBufferSize - idx;
        }
        else
        {
            tempLen = PACKET_SIZE;
        }
        
        pParent->MonStateSet(S_FWLOAD_CMD_SENT);
       	len = CUtil::GetFwProgramPkt (cmdLine, (char*)&(pParent->m_memBuffer[idx]), tempLen, baseAddr+idx);
        pParent->SendStr (cmdLine, len);
        
        idx += tempLen;
        
        if (idx > actualProgress)
        {
            pParent->m_progress.StepIt();
            actualProgress += memBufferSize/100;
        }
        
        // Wait for the ACK
        pParent->MonStateCheck(S_FWLOAD_CMD_ACK, 5, 60);
    }
    pParent->m_progress.SetPos(100);
    
    // The loading has been completed sucessfully
    pParent->MonStateSet(S_FWLOAD_CMD_CMP);	
    
    // Clean up and exit
    pParent->TermFlashLoadingThd();
    return 0;
}

/*-----------------------------------------------------------------------------
Function:       TermFlashLoadingThd
    
Description:    Terminates the programming thread and associated resources for
                each open COM port.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::TermFlashLoadingThd()
{
    m_loading_started = false;
    
    GetDlgItem(IDC_START)->EnableWindow(true);
    
    for (int i=0; i<NUM_PORTS; i++)
    {
        if (m_bool_port_selected[i])
        {
            if(m_commctrl[i].GetPortOpen())
            {
                m_commctrl[i].SetPortOpen(FALSE);
            }
        }
    }
    
    TerminateThread (m_flashLoadingThread, 0);
}

/*-----------------------------------------------------------------------------
Function:       MonStateSet
    
Description:    Sets the application state for all open COM ports.

Parameters:     state - new application state

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::MonStateSet(int state)
{
    for (int i=0; i<NUM_PORTS; i++)
    {
        if (m_bool_port_selected[i] && m_loading_started)
        {
            m_int_mon_state[i] = state;
            m_edit_status[i].SetWindowText(m_str_ok_msg[state]);							
        }
    }
}

/*-----------------------------------------------------------------------------
Function:       MonStateCheck
    
Description:    Checks if the current application state matches the specified
                value with a time period defined by (msecs * retries), and 
                terminates programming if none of the open COM
                ports are in the specified state.

Parameters:     state - a state against which the current state is checked
                msecs - wait time
                retries - number of retries

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::MonStateCheck(int state, int msecs, char retries)
{
    int numOfOpenPorts = 0, i;
    int failCnt=0;
    bool checkAgain[NUM_PORTS];
    bool allDone = false;
    char currentRun = 0;
    
    // initialize which ports to check
    for (i=0;i<NUM_PORTS; i++)
    {
        if (m_bool_port_selected[i] && m_loading_started)
        {
            checkAgain[i] = true;
        }
        else
        {
            checkAgain[i] = false;
        }
    }
    
    while (!allDone)
    {
        failCnt = 0;
        
        // wait/sleep for msecs
        Sleep(msecs);
        
        for (i=0;i<NUM_PORTS; i++)
        {
            if (checkAgain[i] == true)
            {
                if (m_int_mon_state[i] == state)
                {
                    m_edit_status[i].SetWindowText(m_str_ok_msg[state]);							
                    numOfOpenPorts++;
                    checkAgain[i] = false;
                }
                else
                {
                    failCnt++;
                }
            }
        }
        if ((failCnt == 0) || (++currentRun >= retries))
        {
            // done if there is no failure OR enough retries already
            allDone = true;
        }
    }
    
    for (i=0;i<NUM_PORTS; i++)
    {
        if (checkAgain[i] == true)
        {
            CString str;
            str.Format("Error detected with the unit connected to COM %d: %s",
                i+1, m_str_err_msg[state]);
            AfxMessageBox(str, MB_ICONSTOP|MB_OK);
            m_edit_status[i].SetWindowText(m_str_err_msg[state]);
            m_bool_port_selected[i] = false;
            if (m_commctrl[i].GetPortOpen())
            {
                m_commctrl[i].SetPortOpen(FALSE);
            }
        }
    }
    
    if (numOfOpenPorts == 0)
    {
        TermFlashLoadingThd();
    }
}

/*-----------------------------------------------------------------------------
Function:       OnCommEvent_x

Description:    The system calls this function when a COM port event has
                occurred.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnCommEvent_1()
{
    OnCommEventHandler(1);
}
void CFlashLoaderDlg::OnCommEvent_2()
{
    OnCommEventHandler(2);
}
void CFlashLoaderDlg::OnCommEvent_3()
{
    OnCommEventHandler(3);
}
void CFlashLoaderDlg::OnCommEvent_4()
{
    OnCommEventHandler(4);
}
void CFlashLoaderDlg::OnCommEvent_5()
{
    OnCommEventHandler(5);
}
void CFlashLoaderDlg::OnCommEvent_6()
{
    OnCommEventHandler(6);
}
void CFlashLoaderDlg::OnCommEvent_7()
{
    OnCommEventHandler(7);
}
void CFlashLoaderDlg::OnCommEvent_8()
{
    OnCommEventHandler(8);
}
void CFlashLoaderDlg::OnCommEvent_9()
{
    OnCommEventHandler(9);
}
void CFlashLoaderDlg::OnCommEvent_10()
{
    OnCommEventHandler(10);
}
void CFlashLoaderDlg::OnCommEvent_11()
{
    OnCommEventHandler(11);
}
void CFlashLoaderDlg::OnCommEvent_12()
{
    OnCommEventHandler(12);
}
void CFlashLoaderDlg::OnCommEvent_13()
{
    OnCommEventHandler(13);
}
void CFlashLoaderDlg::OnCommEvent_14()
{
    OnCommEventHandler(14);
}
void CFlashLoaderDlg::OnCommEvent_15()
{
    OnCommEventHandler(15);
}
void CFlashLoaderDlg::OnCommEvent_16()
{
    OnCommEventHandler(16);
}

/*-----------------------------------------------------------------------------
Function:       OnCheckX

Description:    The system calls this function when a COM port checkbox is
                selected from the main window.

Parameters:     none

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnCheck1() 
{
    OnCheckHandler(1);
}
void CFlashLoaderDlg::OnCheck2() 
{
    OnCheckHandler(2);
}
void CFlashLoaderDlg::OnCheck3() 
{
    OnCheckHandler(3);
}
void CFlashLoaderDlg::OnCheck4() 
{
    OnCheckHandler(4);
}
void CFlashLoaderDlg::OnCheck5() 
{
    OnCheckHandler(5);
}
void CFlashLoaderDlg::OnCheck6() 
{
    OnCheckHandler(6);
}
void CFlashLoaderDlg::OnCheck7() 
{
    OnCheckHandler(7);
}
void CFlashLoaderDlg::OnCheck8() 
{
    OnCheckHandler(8);
}
void CFlashLoaderDlg::OnCheck9() 
{
    OnCheckHandler(9);
}
void CFlashLoaderDlg::OnCheck10() 
{
    OnCheckHandler(10);
}
void CFlashLoaderDlg::OnCheck11() 
{
    OnCheckHandler(11);
}
void CFlashLoaderDlg::OnCheck12() 
{
    OnCheckHandler(12);
}
void CFlashLoaderDlg::OnCheck13() 
{
    OnCheckHandler(13);
}
void CFlashLoaderDlg::OnCheck14() 
{
    OnCheckHandler(14);
}
void CFlashLoaderDlg::OnCheck15() 
{
    OnCheckHandler(15);
}
void CFlashLoaderDlg::OnCheck16() 
{
    OnCheckHandler(16);
}

/*-----------------------------------------------------------------------------
Function:       OnCheckHandler

Description:    The system calls this function when a COM port checkbox is
                selected from the main window for a specific COM port.

Parameters:     portNum - COM port number

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnCheckHandler (int portNum)
{
    BSTR bstrLicense = ::SysAllocStringLen (pwchLicenseKey, 
        sizeof(pwchLicenseKey)/sizeof(WCHAR));
    
    if (m_checkboxs[portNum-1].GetCheck() == 1)
    {
        if (!m_commctrl_created[portNum-1])
        {
            // Create the MSCOMM32 OLE Control.
            if (!m_commctrl[portNum-1].Create(NULL,0,CRect(0,0,0,0),this,
                ID_COMMCTRL[portNum-1],NULL, false, bstrLicense))
            {
                TRACE0("Failed to create OLE Communications Control\n");
                AfxMessageBox("Failed to create OLE Communications Control\n");
                m_commctrl_created[portNum-1] = false;
                m_bool_port_selected[portNum-1] = false;
            }
            else
            {
                m_commctrl_created[portNum-1] = true;
                m_bool_port_selected[portNum-1] = true;
            }
        }
        
        /* Set the default port settings. Please note that it's very important
        * to set RThreshold property to 1. The default RThreshold value (0)
        * causes the OnComm event to NOT fire when a character is detected in
        * the serial port. 
        */
        CString str;
        str.Format(_T("%d,%C,%d,%d"),9600,(TCHAR)'N',8,1);
        m_commctrl[portNum-1].SetSettings(str);
        m_commctrl[portNum-1].SetRThreshold(1);	
        m_commctrl[portNum-1].SetCommPort(portNum);
        m_commctrl[portNum-1].SetInputMode(1);
        m_commctrl[portNum-1].SetInputLen(0); 
        
        TRY
        {
            m_commctrl[portNum-1].SetPortOpen(TRUE);	
        }
        CATCH_ALL(e)
        {
            char szBuf[4196] = {0};
            CString str;
            e->GetErrorMessage(szBuf, 4195);
            str.Format("Failed to open Serial Port #%d: %s", portNum, szBuf);
            AfxMessageBox(str);
            m_checkboxs[portNum-1].SetCheck(0);
            m_commctrl_created[portNum-1] = false;
        }
        END_CATCH_ALL
    }
    else if(m_checkboxs[portNum-1].GetCheck() == 0)
    {
        if (m_commctrl[portNum-1].GetPortOpen())
        {
            m_commctrl[portNum-1].SetPortOpen(FALSE);
        }
    }
    
    ::SysFreeString(bstrLicense);
}

/*-----------------------------------------------------------------------------
Function:       OnCommEventHandler

Description:    The system calls this function when a COM port event has
                occurred.

Parameters:     portNum - COM port number

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::OnCommEventHandler(int portNum)
{
    CString strEvent, strError;
    
    switch (m_commctrl[portNum-1].GetCommEvent())
    {
        case 1: // vbMSCommEvSend:
            strEvent.LoadString(IDS_SENDING);
            break;
        
        case 2: // vbMSCommEvReceive:
            strEvent.LoadString(IDS_RECEIVING);
            ProcessReceivedData(portNum);
            break;
        
        case 3: // vbMSCommEvCTS:
            strEvent.LoadString(IDS_CTSCHANGE);
            break;
        
        case 4: // vbMSCommEvDSR:
            strEvent.LoadString(IDS_DSRCHANGE);
            break;
        
        case 5: // vbMSCommEvCD:
            strEvent.LoadString(IDS_CDCHANGE);
            break;
        
        case 6: // vbMSCommEvRing:
            strEvent.LoadString(IDS_PHONERING);
            break;
        
        case 7: // vbMSCommEvEOF:
            strEvent.LoadString(IDS_EOFDETECTED);
            break;
        
        case 1001: //  vbMSCommErBreak:
            strError.LoadString(IDS_BREAKRECEIVED);
            break;
        
        case 1002: // vbMSCommErCTSTO:
            strError.LoadString(IDS_CTSTIMEOUT);
            break;
        
        case 1003: // vbMSCommErDSRTO:
            strError.LoadString(IDS_DSRTIMEOUT);
            break;
        
        case 1004: // vbMSCommErFrame:
            strError.LoadString(IDS_FRAMEERROR);
            break;
        
        case 1006: // vbMSCommErOverrun:
            strError.LoadString(IDS_OVERRUNERROR);
            break;
        
        case 1007: // vbMSCommErCDTO:
            strError.LoadString(IDS_CDTIMEOUT);
            break;
        
        case 1008: // vbMSCommErRxOver:
            strError.LoadString(IDS_RBUFFEROVERFLOW);
            break;
        
        case 1009: // vbMSCommErRxParity:
            strError.LoadString(IDS_PARITYERROR);
            break;
        
        case 1010: // vbMSCommErTxFull:
            strError.LoadString(IDS_TBUFFERFULL);
            break;
        
        default:
            break;
    }
}

/*-----------------------------------------------------------------------------
Function:       ProcessReceivedData

Description:    Processed received data for a specified COM port.

Parameters:     portNum - COM port number

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::ProcessReceivedData (int portNum)
{
    COleVariant   var; 
    long          len;
    HRESULT       hr;
    BYTE          *pAccess;
    
    if (!m_bool_port_selected[portNum-1] || !m_loading_started)
    {
        return;
    }
    
    var.Attach(m_commctrl[portNum-1].GetInput());
    
    if ((hr = SafeArrayGetUBound(var.parray,1,&len)) != S_OK)
    {
        return;
    }
    
    len++;
    
    if ((hr = SafeArrayAccessData(var.parray, (void**)&pAccess)) != S_OK)
    {
        return;
    }
    
    for (int ijk=0; ijk<len; ijk++)
    {
        switch (m_int_mon_state[portNum-1])
        {
            case S_MON_INIT:
                break;
            
            default:
                if (pAccess[ijk] == ACK)
                {
                    m_int_mon_state[portNum-1]++;
                }
                break;
        }
    }
    
    SafeArrayUnaccessData(var.parray);
}

/*-----------------------------------------------------------------------------
Function:       SendStr
    
Description:    Sends a specified string on all open COM ports.

Parameters:     str - a pointer to a string to send
                len - length of data in bytes

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::SendStr(char *str, short len)
{
    CByteArray btArray;
    
    for (int i=0; i<len; i++)
    {
        btArray.Add(str[i]);
    }
    COleVariant var(btArray);
    
    for (i=0; i<NUM_PORTS; i++)
    {
        if (m_bool_port_selected[i])
        {
            m_commctrl[i].SetOutput(var);
            m_commctrl[i].SetOutBufferCount(0);	// flush the buffer
        }
    }
}

/*-----------------------------------------------------------------------------
Function:       SetPortParams
    
Description:    Sets the settings of each selected COM port as specified in
                the parameter string.

Parameters:     strParams - COM port parameters

Return Value:   none
-----------------------------------------------------------------------------*/
void CFlashLoaderDlg::SetPortParams(CString strParams)
{
    for (int i=0; i<NUM_PORTS; i++)
    {
        if (m_bool_port_selected[i])
        {
            m_commctrl[i].SetSettings(strParams);
            
            if (!m_commctrl[i].GetPortOpen())
            {
                m_commctrl[i].SetPortOpen(true);
            }
            
            Sleep (2);
        }
    }
}
